@@ -71,17 +71,74 @@ AREF_INTERNAL1V1 = 0x02
 AREF_INTERNAL2V56 = 0x03
 AREF_EXTERNAL = 0x04
 
+import os
+if os.name == 'nt':
+  if sys.version_info[0] == 3:
+    import winreg
+  else:
+    import _winreg as winreg
+
+
+if sys.platform.startswith('linux'):
+  def __FROM(x):
+    return ' find ' + x + ' -maxdepth 0 -print '
+
+  def __SELECT():
+    return " | xargs -I}{ find '}{' "
+
+  def __AND():
+    return __SELECT() + ' -maxdepth 1 '
+
+  def __SUBSYSTEM(x):
+    return " -type l -name subsystem -lname \\*/" + x + " -printf '%h\\n' "
+
+  def __SUBSYSTEMF(x):
+    return " -type l -name subsystem -lname \\*/" +x+ " -printf '%%h\\n' "
+
+  def __SYSATTR(x, y):
+    return " -type f -name " +x+ " -execdir grep -q '" +y+ "' '{}' \\; -printf '%h\\n' "
+
+  def __SYSATTRF(x, y):
+    return " -type f -name " +x+ " -execdir grep -q '" +y+ "' '{}' \\; -printf '%%h\\n' "
+
+  def __FIRST():
+    return " -quit "
+
+  def __SELECTUP():
+    return " | xargs -I}{ sh -c 'x=\"}{\"; while [ \"/\" != \"$x\" ]; do dirname \"$x\"; x=$(dirname \"$x\"); done' " + __AND()
+
+  def findDongle():
+    dongleIDs = [ ('Barobo, Inc.', 'Mobot USB-Serial Adapter'),
+                  ('Barobo, Inc.', 'Linkbot USB-Serial Adapter'),
+                  ('Barobo, Inc.', 'Barobo USB-Serial Adapter') ]
+    import os
+    import subprocess
+    try: 
+      sysfs = os.environ['SYSFS_PATH']
+    except:
+      sysfs = '/sys'
+    for (manufacturer, productid) in dongleIDs:
+      cmd = __FROM(sysfs+'/devices')+__SELECT()+__SYSATTR('manufacturer', manufacturer)+\
+            __AND() + __SYSATTR('product', productid) +\
+            __SELECT() + __SUBSYSTEM('tty') +\
+            " | xargs -I{} grep DEVNAME '{}'/uevent"  +\
+            " | cut -d= -f2"
+      p = subprocess.check_output(['/bin/sh', '-c', cmd])
+      if len(p) > 1:
+        return (str('/dev/')+p.decode('utf-8')).rstrip()
+
 def _getSerialPorts():
   import serial
   if os.name == 'nt':
     available = []
+    handle = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 'HARDWARE\\DEVICEMAP\\SERIALCOMM')
     for i in range(256):
       try:
-        s = serial.Serial(i)
-        available.append('\\\\.\\COM'+str(i+1))
-        s.close()
-      except Serial.SerialException:
-        pass
+        name, port, _ = winreg.EnumValue(handle, i)
+        if name[:14] == '\\Device\\USBSER':
+          available.append(port)
+      except:
+        break
     return available
   else:
     from serial.tools import list_ports
@@ -268,24 +325,20 @@ class BaroboCtx():
   def addLinkbot(self, linkbot):
     self.children.append(linkbot)
 
-  def __autoConnectWorker(self, comport):
-    pass
-
   def autoConnect(self):
-    import threading
-    # Try to connect to all COM ports simultaneously.
-    self.__foundComPort = None
-    self.__foundComPortCond= threading.Condition()
-    myports = _getSerialPorts()
-    self.__workerThreads = []
+    if os.name == 'nt':
+      myports = _getSerialPorts()
+    else:
+      myports = [findDongle()]
+    connected = False
     for port in myports:
-      thread = threading.Thread(target=self.__autoConnectWorker, args=(port))
-      self.__workerThreads.append(thread)
-      thread.daemon = True
-      thread.start()
-    self.foundComPortCond.acquire()
-    while self.__foundComPort == None:
-      self.foundComPortCond.wait()
+      try:
+        self.connectDongleSFP(port)
+        connected = True
+      except:
+        pass
+    if not connected:
+      raise BaroboException('Could not find attached dongle.')
 
   def connect(self):
     """
